# Created by Q-ays.
# whosqays@gmail.com

# install sqlalchemy before use

# postgres tools

from sqlalchemy.exc import SQLAlchemyError, TimeoutError
import traceback
import json


def session_exception(session, is_raise=True):
    def wrapper(func):

        def catch(*args, **kwargs):
            try:
                result = func(*args, **kwargs)
                return result
            except SQLAlchemyError as e:
                # print(e)
                print('~~~~~~~~~~~~~~~~session error~~~~~~~~~~~~~~~~~~')
                traceback.print_exc()
                if not isinstance(e, TimeoutError):
                    session.rollback()
                if is_raise:
                    raise e

        return catch

    return wrapper


def to_dict(self):
    return {c.name: getattr(self, c.name, None) for c in self.__table__.columns}


def str2js(ls):
    """
    json字符转字典
    :param ls:
    :return:
    """
    if isinstance(ls, list):
        ls0 = list()

        try:
            for l in ls:
                ls0.append(json.loads(l))

            return ls0
        except:
            pass

    return ls


def detect_filed(o, *args):
    """
    特定字段字符转json
    :param o:
    :param args:
    :return:
    """
    for arg in args:
        n = o.get(arg)
        if n:
            o[arg] = str2js(n)

    return o


def o2d(obj):
    """
    把对象(支持单个对象、list、set)转换成字典
    针对postgres数据库
    :param obj: obj, list, set
    :return:
    """

    if isinstance(obj, dict) or (not obj):
        return obj

    is_list = isinstance(obj, list)
    is_set = isinstance(obj, set)

    if is_list or is_set:
        obj_arr = []

        for o in obj:
            if o:
                if isinstance(o, dict):
                    n = detect_filed(o, 'roles', 'org')
                else:
                    n = detect_filed(o.to_dict(), 'roles', 'org')

                obj_arr.append(n)
        return obj_arr
    else:
        return detect_filed(obj.to_dict(), 'roles', 'org')


def repo_ref(session0):
    """
    基于postgres 数据库增删改查的公共类
    :param session0:
    :return:
    """

    class RepoBase:
        def __init__(self, Model=None):
            self.session = session0
            self.Model = Model

        def add(self, **data):
            model0 = self.Model(**data)

            self.session.add(model0)
            self.session.commit()

            return model0

        def delete(self, did=None, **kwargs):
            did = did if did else kwargs.get('id')
            if did:
                repo = self.session.query(self.Model).get(did)

                self.session.delete(repo)
                self.session.commit()

                return repo
            else:
                if kwargs:
                    repos = self.session.query(self.Model).filter_by(**kwargs)
                    repos_r = repos.all()
                    repos_d = repos.delete()

                    self.session.commit()

                    return repos_r
            raise Exception('未传参数，删除规则')

        def update(self, did=None, **data):

            did = did if did else data.get('id')

            if not did:
                raise Exception('更新的数据没得id，不晓得更新哪条')

            model0 = self.get(did)

            columns = model0.__table__.columns

            for col in columns:
                name = col.name
                value = data.get(name, None)

                if value is not None and not col.primary_key:
                    setattr(model0, name, value)

            # 统一修改密码
            if data.get('password'):
                model0.password = data.get('password')

            self.session.add(model0)
            self.session.commit()

            return model0

        def get(self, did=None, _fuzzy=None, _sort=None, _slice=None, **kwargs):
            repos = self.session.query(self.Model)

            if did:
                if isinstance(did, list):
                    repos = repos.filter(self.Model.id.in_(did))
                else:
                    repos = repos.get(did)

                    # self.session.close()
                    self.session.commit()

                    return repos
            elif kwargs:
                if _fuzzy:
                    for (k, v) in kwargs.items():
                        if isinstance(v, list):
                            if len(v) == 2:
                                repos = repos.filter(getattr(self.Model, k).between(*v))
                        else:
                            repos = repos.filter(getattr(self.Model, k).like('%' + v + '%'))
                else:
                    repos = repos.filter_by(**kwargs)
            else:
                ...

            if _sort:
                if _sort.startswith('-'):
                    repos = repos.order_by(getattr(self.Model, _sort[1:]).desc())
                else:
                    repos = repos.order_by(getattr(self.Model, _sort))

            if _slice:
                repos = repos.slice(*_slice)

            repos = repos.all()
            # self.session.close()
            self.session.commit()

            return repos

        def page(self, did=None, _fuzzy=None, _sort=None, _slice=None, **kwargs):
            """
            paginate and return count total and current slice
            :param did:
            :param _fuzzy:
            :param _sort:
            :param _slice:
            :param kwargs:
            :return:
            """

            repos = self.session.query(self.Model)

            if did:
                if isinstance(did, list):
                    repos = repos.filter(self.Model.id.in_(did))
                else:
                    repos = repos.get(did)

                    # self.session.close()
                    self.session.commit()

                    return repos
            elif kwargs:
                if _fuzzy:
                    for (k, v) in kwargs.items():
                        if isinstance(v, list):
                            if len(v) == 2:
                                repos = repos.filter(getattr(self.Model, k).between(*v))
                        else:
                            repos = repos.filter(getattr(self.Model, k).like('%' + v + '%'))
                else:
                    repos = repos.filter_by(**kwargs)
            else:
                ...

            total = repos.count()

            if _sort:
                if _sort.startswith('-'):
                    repos = repos.order_by(getattr(self.Model, _sort[1:]).desc())
                else:
                    repos = repos.order_by(getattr(self.Model, _sort))

            if _slice:
                repos = repos.slice(*_slice)

            repos = repos.all()
            # self.session.close()
            self.session.commit()

            return {'total': total, 'data': o2d(repos), 'page': _slice}

        def create_view(self, sql_create_view):
            '''
            创建视图函数（视图创建使用后便删除）
            如要查询用户创建的组织信息和用户信息
            select a.account, a.username, row_to_json(o) as org
            from public.user as a, public.organization as o
            where o.owner=a.id
            :param sql_create_view:
            :return: 视图表数据
            '''
            create_view = "create view tzxdview as {}".format(sql_create_view)
            sql_2 = "select * from tzxdview"
            try:
                self.session.execute(create_view)
            except:
                self.session.close()
                raise Exception('语句执行错误！')

            res = self.session.execute(sql_2)
            self.session.execute("drop view tzxdview")
            self.session.close()
            res = [dict(r) for r in res]
            return res

    return RepoBase



# Used for micro-service which developed by nameko
# install nameko before use
"""
    Example::

        from wisdoms.auth import permit

        host = {'AMQP_URI': "amqp://guest:guest@localhost"}

        auth = permit(host)

        class A:
            @auth
            def func():
                pass
"""
from nameko.rpc import rpc
from nameko.standalone.rpc import ServiceRpcProxy
from nameko.exceptions import RpcTimeout, MethodNotFound

import json
import traceback

from functools import wraps
from operator import methodcaller

from wisdoms.utils import xpt_func
from wisdoms.exec import UserException, WisdomErrorException
from wisdoms.commons import revert, codes

default_host = {'AMQP_URI': 'pyamqp://guest:guest@localhost'}
default_base_service = 'baseUserApp'


def rpc_with_timeout(host, service, func, data=None, timeout=8):
    try:
        with ServiceRpcProxy(service, host, timeout=timeout) as proxy:
            if data is not None:
                res = methodcaller(func, data)(proxy)
            else:
                res = methodcaller(func)(proxy)
            return res
    except RpcTimeout as e:
        print(service, ' ~~连接超时 %s sec......，检查是否启动......' % e)
        return revert(codes.TIMEOUT, desc=service + ' 连接超时')
    except MethodNotFound as e:
        print('function of this server %s not found,未找到方法 %s ' % (service, e))
        return revert(codes.GET_WAY_ERROR, desc='未找到方法' + e)


def rpc_wrapper(service_, func_, *args, host_=default_host, timeout_=8, **kwargs):
    """
    rpc包装方法

    :param service_:
    :param func_:
    :param host_:
    :param timeout_:
    :param args:
    :param kwargs:
    :return:
    """
    try:
        with ServiceRpcProxy(service_, host_, timeout=timeout_) as proxy:
            res = methodcaller(func_, *args, **kwargs)(proxy)
            return res
    except RpcTimeout as e:
        print(service_, ' ~~连接超时 %s sec......，检查是否启动......' % e)
        return revert(codes.TIMEOUT, desc=service_ + ' 连接超时')
    except MethodNotFound as e:
        print('function of this server %s not found,未找到方法 %s ' % (service_, e))
        return revert(codes.GET_WAY_ERROR, desc='未找到方法' + e)


def ms_base(ms_host=None, base_service=None, func=None, **extra):
    """
    返回父类，闭包，传参数ms host
    :param ms_host:
    :param base_service
    :param func
    :param extra: 额外信息
    :extra: roles 角色权限
    :extra: name 微服务名称
    :extra: types 微服务类型 essential 类型的应用 角色必须带creator
    :extra: entrance 微服务pc入口
    :extra: entrance4app 小程序入口
    :extra: entrance4back 后台入口
    :extra: identity 唯一标识 暂时只用于平台base service 其他微服务不添加
    :return: class of base
    """

    ms_host = ms_host if ms_host else default_host
    base_service = base_service if base_service else default_base_service
    func = func if func else 'app2db'

    class MsBase:
        name = 'ms-base'

        @rpc
        # @exception()
        def export_info2db(self):
            """
            export information of this service to database

            :return:
            """
            clazz = type(self)
            service = clazz.name
            functions = dir(clazz)

            origin = extra
            origin['service'] = service
            origin['functions'] = functions

            rpc_with_timeout(ms_host, base_service, func, origin)

    return MsBase


def permit(host=None, **extra):
    """
    调用微服务功能之前，进入基础微服务进行权限验证

    :param: host: micro service host
    :extra: service 基础应用微服务名称
    :extra: func 用户验证方法名称
    :extra: full_info 是否携带用户和组织全部信息 默认false
    :return decorator
    """
    host = host if host else default_host
    base_service = extra.get('service', default_base_service)
    verify_func = extra.get('func', 'verify')

    full_info = extra.get('full_info', False)

    def wrapper(f):
        @wraps(f)
        def inner(*args, **kwargs):
            service = args[0].name
            func = f.__name__
            token = args[1].get('token')

            res = rpc_with_timeout(host, base_service, verify_func, {'service': service, 'func': func, 'token': token})

            try:
                # judge except
                if res.get('code'):
                    if res.get('code') < 0:
                        return res
            except:
                pass

            if res:
                org = None
                try:
                    # 添加组织信息 add info of org
                    org = res.get('org').get(str(res.get('current_org')))
                except:
                    pass

                try:
                    # delete user passoword
                    del res['password_hash']
                    del res['partner']
                except:
                    pass

                cut_user = dict()
                cut_org = dict()

                cut_user['id'] = res['id']
                cut_user['account'] = res['account']
                cut_user['username'] = res.get('username')
                cut_user['current_org'] = res.get('current_org')
                cut_user['phone'] = res.get('phone')
                cut_user['recommend_phone'] = res.get('recommend_phone')
                cut_user["agent_oid"] = res.get("agent_oid")
                cut_user["from_agent"] = res.get("from_agent")
                cut_user["member_phone"] = res.get("member_phone")
                cut_user['email'] = res.get('email')
                cut_user['appid'] = res.get('appid')
                cut_user['openid'] = res.get('openid')
                try:
                    if org:
                        cut_org['id'] = org['id']
                        cut_org['name'] = org['name']
                        cut_org['desc'] = org.get('desc')
                        cut_org['owner'] = org.get('owner')
                        cut_org['region'] = org.get('region')
                        cut_org['es_extend'] = org.get('es_extend')
                        cut_org['share_profit_id'] = org.get('share_profit_id')
                        cut_org['mch_id'] = org.get('mch_id')
                        cut_org['sp_appid'] = org.get('sp_appid')
                        cut_org['appid'] = org.get('appid')
                        cut_org['images'] = org.get('images')
                        cut_org['videos'] = org.get('videos')
                        cut_org['location'] = org.get('location')
                except:
                    pass

                if full_info:
                    args[1]['full_user'] = res
                    args[1]['full_org'] = org

                args[1]['uid'] = res.get('id')
                args[1]['user'] = cut_user
                args[1]['org'] = cut_org

                return f(*args, **kwargs)

            raise UserException('verified failed-用户权限验证失败')

        return inner

    return wrapper


def add_uid(host=None, **extra):
    """
    用户token 返回用户id
    :param host:
    :extra: service 基础应用微服务名称
    :extra: func 获取用户uid方法名称
    :return: decorator
    """
    host = host if host else default_host
    base_service = extra.get('service', default_base_service)
    get_uid_func = extra.get('func', 'get_uid')

    def wrapper(f):
        @wraps(f)
        def inner(*args, **kwargs):
            token = args[1].get('token')

            res = rpc_with_timeout(host, base_service, get_uid_func, {'token': token})

            try:
                if res.get('code'):
                    if res.get('code') < 0:
                        return res
            except:
                pass

            if res:
                args[1]['uid'] = res
                return f(*args, **kwargs)

            raise UserException('verified failed-用户token验证失败')

        return inner

    return wrapper


def add_user(host=None, **extra):
    """
    用户token 返回用户信息
    :param host:
    :extra: service 基础应用微服务名称
    :extra: func 获取用户信息方法名称
    :extra: full_info 是否携带用户和组织全部信息 默认false
    :return: decorator
    """

    host = host if host else default_host
    base_service = extra.get('service', default_base_service)
    get_user_func = extra.get('func', 'get_user')
    full_info = extra.get('full_info', False)

    def wrapper(f):
        @wraps(f)
        def inner(*args, **kwargs):
            token = args[1].get('token')

            res = rpc_with_timeout(host, base_service, get_user_func, {'token': token})

            try:
                if res.get('code'):
                    if res.get('code') < 0:
                        return res
            except:
                pass

            if res:

                org = None
                try:
                    # 添加组织信息 add info of org
                    org = res.get('org').get(str(res.get('current_org')))
                except:
                    pass

                try:
                    # delete user passoword
                    del res['password_hash']
                    del res['partner']
                except:
                    pass

                cut_user = dict()
                cut_org = dict()

                cut_user['id'] = res['id']
                cut_user['account'] = res['account']
                cut_user['username'] = res.get('username')
                cut_user['current_org'] = res.get('current_org')
                cut_user['phone'] = res.get('phone')
                cut_user['recommend_phone'] = res.get('recommend_phone')
                cut_user["agent_oid"] = res.get("agent_oid")
                cut_user["from_agent"] = res.get("from_agent")
                cut_user["member_phone"] = res.get("member_phone")
                cut_user['email'] = res.get('email')
                cut_user['avatar'] = res.get('avatar')
                cut_user['appid'] = res.get('appid')
                cut_user['openid'] = res.get('openid')
                try:
                    if org:
                        cut_org['id'] = org['id']
                        cut_org['name'] = org['name']
                        cut_org['desc'] = org.get('desc')
                        cut_org['owner'] = org.get('owner')
                        cut_org['region'] = org.get('region')
                        cut_org['es_extend'] = org.get('es_extend')
                        cut_org['share_profit_id'] = org.get('share_profit_id')
                        cut_org['mch_id'] = org.get('mch_id')
                        cut_org['sp_appid'] = org.get('sp_appid')
                        cut_org['appid'] = org.get('appid')
                        cut_org['images'] = org.get('images')
                        cut_org['videos'] = org.get('videos')
                        cut_org['location'] = org.get('location')
                except:
                    pass

                if full_info:
                    args[1]['full_user'] = res
                    args[1]['full_org'] = org

                args[1]['uid'] = res.get('id')
                args[1]['user'] = cut_user
                args[1]['org'] = cut_org

                return f(*args, **kwargs)

            raise UserException('verified failed-用户token验证失败')

        return inner

    return wrapper


def privilege(host=None, **extra):
    """

    :param host:
    :param extra:
    :extra: service 基础应用微服务名称
    :extra: func 验证超级用户
    :extra: level 权限级别：default = 10
    :return:
    """
    host = host if host else default_host
    base_service = extra.get('service', default_base_service)
    get_user_func = extra.get('func', 'get_user')
    level = extra.get('level', 10)

    def wrapper(f):
        @wraps(f)
        def inner(*args, **kwargs):
            token = args[1].get('token')

            res = rpc_with_timeout(host, base_service, get_user_func, {'token': token})

            try:
                if res.get('code'):
                    if res.get('code') < 0:
                        return res
            except:
                pass

            if res:
                try:
                    del res['password_hash']
                    del res['partner']
                except:
                    pass

                #  验证用户特权级别
                p = res.get('privilege', None)
                if p:
                    if p.get('level', 0) >= level:
                        args[1]['user'] = res
                        return f(*args, **kwargs)

            raise UserException('verified failed-用户特殊权限验证失败，用户特殊权限，或权限级别不够。权限级别:' + str(level))

        return inner

    return wrapper


def agent(host=None, **extra):
    """
    用户token 返回用户信息
    :param host:
    :extra: service 基础应用微服务名称
    :extra: func 获取用户信息方法名称
    :return: decorator
    """

    host = host if host else default_host
    base_service = extra.get('service', default_base_service)
    get_user_func = extra.get('func', 'get_agent')

    def wrapper(f):
        @wraps(f)
        def inner(*args, **kwargs):
            token = args[1].get('token')

            res = rpc_with_timeout(host, base_service, get_user_func, {'token': token})

            try:
                if res.get('code'):
                    if res.get('code') < 0:
                        return res
            except:
                pass

            if res:

                org = None
                try:
                    # 添加组织信息 add info of org
                    org = res.get('org').get(str(res.get('current_org')))
                except:
                    pass

                try:
                    # delete user passoword
                    del res['password_hash']
                    del res['partner']
                except:
                    pass

                cut_user = dict()
                cut_org = dict()

                cut_user['id'] = res['id']
                cut_user['account'] = res['account']
                cut_user['username'] = res.get('username')
                cut_user['current_org'] = res.get('current_org')
                cut_user['phone'] = res.get('phone')
                cut_user['email'] = res.get('email')

                try:
                    if org:
                        cut_org['id'] = org['id']
                        cut_org['name'] = org['name']
                        cut_org['desc'] = org.get('desc')
                        cut_org['owner'] = org.get('owner')
                        cut_org['region'] = org.get('region')
                        cut_org['es_extend'] = org.get('es_extend')
                except:
                    pass

                tmp_permit = dict()

                tmp_permit['privilege'] = res.get('privilege')
                tmp_permit['org'] = res.get('agent_org')
                tmp_permit['region'] = res.get('agent_region')
                tmp_permit['agent'] = res.get('agent_sub')

                args[1]['uid'] = res.get('id')
                args[1]['user'] = cut_user
                args[1]['org'] = cut_org
                args[1]['permit'] = tmp_permit

                return f(*args, **kwargs)

            raise UserException('verified failed-用户token验证失败')

        return inner

    return wrapper


def config_server(host=default_host, service='configServerFunc', func='import_config', timeout=30):
    """
    从配置微服务获取配置
    :param host:
    :param service:
    :param func:
    :param timeout:
    :return:
    """
    try:
        with ServiceRpcProxy(service, host, timeout=timeout) as proxy:
            res = methodcaller(func)(proxy)
            data = json.dumps(res, indent=4, ensure_ascii=False, sort_keys=False, separators=(',', ':'))
            print(data)
            return res
    except RpcTimeout as e:
        print('配置中心微服务连接超时，或未启动, 时长:%s' % e)
        return {}
        # return {'code': -30, 'desc': '配置中心微服务连接超时，或未启动, 时长:%s' % e, 'data': None}


def my_rpc(func):
    @rpc
    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            traceback.print_exc()
            print(e)
            if isinstance(e, WisdomErrorException):
                return e.revert()
            return revert(codes.ERROR, desc='其他异常类错误')

    return wrapper


class MsFuncFactory:
    """
    微服务方法工厂class
    统一管理rabbit mq host
    """

    def __init__(self, host):
        self.host = host

    def rpc_wrapper(self, service, func, *args, **kwargs):
        """
        rpc包装器
        :param service:
        :param func:
        :param args:
        :param kwargs:
        :return: result of process of rpc
        """
        return rpc_wrapper(service, func, *args, self.host, **kwargs)

    def ms_base(self):
        """
        应用微服务基础类，父类
        :return: class
        """
        return ms_base(self.host)

    def permit(self, **extra):
        """
        权限验证
        :param extra:
        :return: decorator
        """
        return permit(self.host, **extra)

    def add_user(self, **extra):
        """
        user
        :param extra:
        :return: decorator
        """
        return add_user(self.host, **extra)

    def add_uid(self, **extra):
        """
        user id
        :param extra:
        :return: decorator
        """
        return add_uid(self.host, **extra)

    def privilege(self, **extra):
        """
        平台管理员权限验证装饰器
        :param extra:
        :return: decorator
        """
        return privilege(self.host, **extra)

    def config_server(self):
        """
        从配置中心获取配置
        :return: result type of dict
        """
        return config_server(self.host)



import json
from dapr.client import DaprClient
def dapr_invoke(app, method, data, http_verb="post", **kwargs):
    """
    dapr http invoke
    
    :param app: app_id
    :param method: 方法名称
    :http_verb: http请求方法
    :return:
    """
    with DaprClient() as d:
        res = d.invoke_method(app, method, json.dumps(data), http_verb=http_verb, **kwargs)
        return res


# Created by Q-ays.
# whosqays@gmail.com

from wisdoms.commons import revert, codes
from inspect import ismethod, isfunction
import traceback
import os
import datetime


def joint_base2(url1, url2, char='/'):
    url1 = str(url1)
    url2 = str(url2)

    if char == '':
        return url1 + url2

    if url1.endswith(char) and not url2.startswith(char):
        return url1 + url2
    elif not url1.endswith(char) and url2.startswith(char):
        return url1 + url2
    elif url1.endswith(char) and url2.startswith(char):
        return url1 + url2[1:]
    else:
        return url1 + char + url2


def joint4path(*args, char='/'):
    """
    连接n个路径
    :param args:a,b,c,d
    :param char character
    :return: a/b/c/d
    """
    url1 = args[0]

    length1 = len(args)
    if length1 > 1:
        for i in range(1, length1):
            url1 = joint_base2(url1, args[i], char=char)

    return url1


def generate_str(prefix='', postfix=''):
    """
    生成日期字符串
    :param prefix:
    :param postfix:
    :return:
    """
    date = datetime.datetime.now()
    date_str = datetime.datetime.strftime(date, "%y%m%d-%H%M%S-%f")
    try:
        return prefix + date_str + postfix
    except Exception as e:
        print(e)

    return date_str


def generate_filename(filename):
    """
    生成文件名tzxd
    :param filename:
    :return:
    """
    return generate_str('tzxd', os.path.splitext(filename)[1])


def num2str_mk(num, bit=2):
    return '0' + str(num) if num < pow(10, bit - 1) else str(num)


def ymd(d=None, types='tuple', days=0):
    """
    把一个日期的年月日分成字符串，位数补齐 2020 02 02
    :param d:
    :param types:
    :param days: 正负天数，相对于今天
    :return:
    """
    if d:
        year = str(d.year)
        month = '0' + str(d.month) if d.month < 10 else str(d.month)
        day = '0' + str(d.day) if d.day < 10 else str(d.day)
    else:
        dt = datetime.date.today() + datetime.timedelta(days=days)

        year = str(dt.year)
        month = '0' + str(dt.month) if dt.month < 10 else str(dt.month)
        day = '0' + str(dt.day) if dt.day < 10 else str(dt.day)

    if types == 'dict':
        return {'year': year, 'month': month, 'day': day}
    else:
        return year, month, day


def hms():
    now = datetime.datetime.now()

    h = num2str_mk(now.hour)
    m = num2str_mk(now.minute)
    s = num2str_mk(now.second)
    ms = num2str_mk(now.microsecond, 6)

    return h, m, s, ms


def o2d(obj):
    """
    把对象(支持单个对象、list、set)转换成字典
    :param obj: obj, list, set
    :return:
    """
    is_list = isinstance(obj, list)
    is_set = isinstance(obj, set)

    if is_list or is_set:
        obj_arr = []
        for o in obj:
            # 把Object对象转换成Dict对象
            if o:
                dict1 = {}
                dict1.update(o.__dict__)
                obj_arr.append(dict1)
        return obj_arr
    else:
        dict1 = {}
        dict1.update(obj.__dict__)
        return dict1


def func_exception(code=codes.ERROR):
    """
    捕获方法异常装饰器
    :param code:
    :return:
    """

    def func_wrapper(func):
        def catch(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except Exception as e:
                # print('~~~~~~~~~~~~~~~~~~~~ ', e, ' ~~~~~~~~~~~~~~~~~~~~~~~~~')
                traceback.print_exc()

                return revert(code, e)

        return catch

    return func_wrapper


def dec4method(xpt):
    """
    类装饰器，作用于实例下面的所有方法。
    注意如果是静态方法，用类名调用方法名会报错，必须用实例调用方法名。
    该装饰器直接作用于子类。
    :param xpt: 方法装饰器
    :return:
    """

    def cls_wrapper(cls):

        class Wrapper(object):

            def __init__(self, *args, **kwargs):
                self.wrapper = cls(*args, **kwargs)

            def __getattr__(self, item):

                res = getattr(self.wrapper, item)

                if ismethod(res) or isfunction(res):

                    @xpt
                    def func(*args, **kwargs):
                        return res(*args, **kwargs)

                    return func
                else:
                    return res

        return Wrapper

    return cls_wrapper


def cls_exception(xpt):
    """
    类装饰器，捕获类生成实例方法下面所有异常。
    注意如果是静态方法，用类名调用方法名会报错，必须用实例调用方法名。
    该装饰器直接作用于子类。
    :param xpt: 方法装饰器
    :return:
    """

    return dec4method(xpt)


xpt_func = func_exception()
xpt_cls = cls_exception(xpt_func)

if __name__ == '__main__':
    d = datetime.datetime.now()
    # print(d.hour)
    # print(d.minute)
    # print(d.second)
    # print(d.microsecond)
    print(ymd())
    print(hms())
    print(joint4path(*ymd(), *hms(), char=''))



# Created by Q-ays.
# whosqays@gmail.com


import os
from wisdoms.commons import my_code as codes

SECRET_KEY = os.environ.get('SECRET_KEY') or 'hard to guess string'

revert = codes.revert
codes = codes


ABC_KEYS = ["A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "M", "L", "N", "O", "P", "Q", "R", "S", "T", "U",
            "V", "W", "X", "Y", "Z"]


def success(data=None):
    """
    封装revert方法，返回执行成功code

    :param data:
    :return:
    """
    return revert(codes.SUCCESS, data)



# Created by Q-ays.
# whosqays@gmail.com

from wisdoms.ms import rpc_wrapper

"""
archive = archive_config('announce', 'announcement', {'name': '公告服务', 'service': 'announcementService'}, MS_HOST)

@rpc
@user_
def announce_add(self, origin):
    data = origin.get('data')

    data['create_time'] = datetime.now()
    data['addTime_'] = datetime.now()

    announce = ar.add(data)

    archive(origin, '添加公告', o2d(announce))

    return success2d(announce)

传入origin自动获取用户信息，组织信息，请求数据

"""


def archive_config(archive_index, form_index, service, host):
    """
    存档配置生成
    :param archive_index:存档index
    :param form_index: 表单index
    :param service: 项目微服务信息配置
    :param host: rabbit mq 配置
    :return:
    """
    origin = dict()
    origin['index'] = archive_index
    data = dict()
    data['service'] = service
    data['index'] = form_index
    origin['data'] = data

    def archive_save(info_: dict = None, desc_: str = None, resp_data_=None, event_=True, struct_=False, **kwargs):
        """
        :param info_: 原始信息结构，带有用户信息
        :param desc_: 操作描述
        :param resp_data_: 返回数据
        :param event_: 发送事件的方式存储
        :param struct_: 返回存储数据结构
        :param kwargs: 存档数据结构，可自动添加额外信息
        :return:
        """
        if isinstance(info_, dict):
            data['user'] = info_.get('user')
            data['org'] = info_.get('org')
            data['req_data'] = info_.get('data')
            if isinstance(data['req_data'], dict):
                data['order_id'] = data['req_data'].get('order_')
        data['desc'] = desc_
        data['resp_data'] = resp_data_
        data.update(kwargs)

        if struct_:
            return {'data': origin}

        if event_:
            try:
                res = rpc_wrapper('archivesFunc', 'dispatch', {'data': origin}, host_=host, timeout_=2)
                return res
            except Exception as e:
                print(e)
                print('archive service error')
                return 'archive service error'
        else:
            try:
                res = rpc_wrapper('archivesFunc', 'add', {'data': origin}, host_=host, timeout_=8)
                return res
            except Exception as e:
                print(e)
                print('archive service error')
                return 'archive service error'

    return archive_save
